//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

/* 
 * ORG.opengroup.moabatch.PropertyEditorAccess version 0.1
 * by Bill la Forge
 * http://www.opengroup.org/~laforge
 */

package ORG.opengroup.beans;

import java.beans.*;
import java.lang.reflect.*;

/**
 * Determines if a property is Read/Write and has an associated property editor.
 * Supports indexed properties.
 */
public class PropertyEditorAccess
{
    /**
     * The object being edited.
     */
    Object object = null;

    /**
     * The BeanInfo for the object being edited.
     */
    BeanInfo bi = null;

    /**
     * The property descriptors of the object being edited.
     */
    PropertyDescriptor propertyDescriptors[] = null;

    /**
     * The property descriptor of the selected property.
     */
    PropertyDescriptor propertyDescriptor = null;

    /**
     * The indexed property descriptor of the selected property.
     */
    IndexedPropertyDescriptor indexPropertyDescriptor = null;

    /**
     * The IndexedPropertyDescriptor class
     */
    Class clazz=null;

    /**
     * The type of property.
     */
    Class type = null;

    /**
     * The type of indexed property.
     */
    Class indexType = null;

    /**
     * The property read method.
     */
    Method getter = null;

    /**
     * The indexed property read method.
     */
    Method indexGetter = null;

    /**
     * The property write method.
     */
    Method putter = null;

    /**
     * The indexed property write method.
     */
    Method indexPutter = null;

    /**
     * The property editor.
     */
    PropertyEditor editor = null;

    /**
     * The indexed property editor.
     */
    PropertyEditor indexEditor = null;

    /**
     * Select the object to be edited.
     * @exception IntrospectionException unable to access BeanInfo.
     */
    public PropertyEditorAccess(Object v) 
        throws IntrospectionException,
               ClassNotFoundException
    {
        if (clazz==null)
            clazz=Class.forName("java.beans.IndexedPropertyDescriptor");

        object=v;
        bi = Introspector.getBeanInfo(object.getClass());
        propertyDescriptors = bi.getPropertyDescriptors();
        propertyDescriptor=null;
        editor = null;
        indexEditor = null;
    }

    /**
     * Returns the BeanInfo of the object to be edited.
     */
    public BeanInfo getBeanInfo() 
    {
        return bi;
    }

    /**
     * Returns the Property Descriptors of the object to be edited.
     */
    public PropertyDescriptor[] getPropertyDescriptors() 
    {
        return propertyDescriptors;
    }

    /**
     * Determine the editor of the property
     */
    private void analyzeProperty()
    {
        type = propertyDescriptor.getPropertyType();
        if (type==null)
            return;

        getter = propertyDescriptor.getReadMethod();
        putter = propertyDescriptor.getWriteMethod();
        if (getter==null || putter==null)
            return;

        Class pec = propertyDescriptor.getPropertyEditorClass();
        if (pec != null) 
        {
            try 
            {
                editor = (PropertyEditor)pec.newInstance();
            } catch (Exception ex) 
            {
                //Drop through.
            }
        }
        if (editor == null) 
            editor = PropertyEditorManager.findEditor(type);
    }

    /**
     * Determine the editor of the indexed property
     */
    private void analyzeIndexProperty()
    {
        indexType = indexPropertyDescriptor.getIndexedPropertyType();
       if (indexType==null)
            return;
        indexGetter = indexPropertyDescriptor.getIndexedReadMethod();
        indexPutter = indexPropertyDescriptor.getIndexedWriteMethod();
        if (indexGetter==null || indexPutter==null)
            return;
        indexEditor = PropertyEditorManager.findEditor(indexType);
    }

    /**
     * Select a property on the object.
     * Returns true if the property has an editor.
     */
    public boolean selectProperty(String v)
    {
        int ndx;
        for (ndx=0;ndx<propertyDescriptors.length;++ndx)
            if (v.equalsIgnoreCase(propertyDescriptors[ndx].getDisplayName()))
                return selectProperty(ndx);
        propertyDescriptor=null;
        editor = null;
        indexEditor = null;
        return false;
    }

    /**
     * Select a property on the object.
     * Returns true if the property has an editor.
     */
    public boolean selectProperty(int ndx)
    {
        propertyDescriptor=null;
        editor = null;
        indexEditor = null;
        propertyDescriptor=propertyDescriptors[ndx];
        analyzeProperty();

        if (!Beans.isInstanceOf(propertyDescriptor,clazz))
            return editor!=null;
        indexPropertyDescriptor=
            (IndexedPropertyDescriptor)Beans.getInstanceOf(
                propertyDescriptor,clazz);
        analyzeIndexProperty();
        return editor!=null || indexEditor!=null;
    }

    /**
     * Returns the Property Descriptor of the selected property, or null.
     */
    public PropertyDescriptor getPropertyDescriptor() 
    {
        return propertyDescriptor;
    }

    /**
     * Returns the Property Editor of the selected property, or null.
     */
    public PropertyEditor getPropertyEditor() 
    {
        return editor;
    }

    /**
     * Returns the Indexed Property Editor of the selected property, or null.
     */
    public PropertyEditor getIndexedPropertyEditor() 
    {
        return indexEditor;
    }

    /**
     * Get the current value of the property as a text string.
     * If null is returned, then the property can not be edited as a string.
     */
    public String getValue()
        throws IllegalAccessException, InvocationTargetException
    {
        if (editor==null)
            return null;
        String init = null;
        Object gargs[] = { };
        Object value = getter.invoke(object, gargs);
        editor.setValue(value);
        return editor.getAsText();
    }

    /**
     * Set the current value of the property as a text string.
     */
    public void setValue(String v)
        throws IllegalAccessException, InvocationTargetException
    {
        editor.setAsText(v);
        Object value = editor.getValue();
        Object sargs[] = { value };
        putter.invoke(object,sargs);
    }

    /**
     * Get the current value of the indexed property as a text string.
     * If null is returned, then the property can not be edited as a string.
     */
    public String getValue(int n)
        throws IllegalAccessException, 
               InvocationTargetException, 
               ArrayIndexOutOfBoundsException
    {
        if (indexEditor==null)
            return null;
        Integer indx = new Integer(n);
        //String init = null;
        Object gargs[] = { indx };
        Object value = indexGetter.invoke(object, gargs);
        indexEditor.setValue(value);
        return indexEditor.getAsText();
    }

    /**
     * Set the current value of the indexed property as a text string.
     */
    public void setValue(int n, String v)
        throws IllegalAccessException, 
               InvocationTargetException, 
               ArrayIndexOutOfBoundsException
    {
        Integer indx = new Integer(n);
        indexEditor.setAsText(v);
        Object value = indexEditor.getValue();
        Object sargs[] = { indx, value };
        indexPutter.invoke(object,sargs);
    }
}  
